<?php
// Copyright (C) <2018>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, version 3 of the License.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// 2.
//  If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//  under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//  License agreement and license key will be shipped with the order
//  confirmation.

namespace itnovum\openITCOCKPIT\Checks\Receiver\Plugins;


use itnovum\openITCOCKPIT\Checks\Receiver\Converter\Byte;
use itnovum\openITCOCKPIT\Checks\Receiver\NagiosService;
use itnovum\openITCOCKPIT\Checks\Receiver\ValueObjects\PerfdataGauge;

abstract class BasePlugin implements PluginInterface {

    /**
     * @var array
     */
    protected $pluginConfig = [];

    /**
     * @var array
     */
    protected $json;

    /**
     * @var string
     */
    protected $output = '';

    /**
     * @var string
     */
    protected $longOutput = '';

    /**
     * @var int
     */
    protected $statusCode = 3;

    /**
     * @var array
     */
    private $perfdata = [];

    /**
     * BasePlugin constructor.
     * @param array $pluginConfig
     * @param array $json
     */
    public function __construct($pluginConfig, $json) {
        $this->pluginConfig = $pluginConfig;
        $this->json = $json;
        $this->execute();
    }

    /**
     * @param $statusCode
     * @return $this
     */
    protected function setStatuscode($statusCode) {
        $this->statusCode = (int)$statusCode;
        return $this;
    }

    /**
     * @param $output
     * @return $this
     */
    protected function setOutput($output) {
        $this->output = $output;
        return $this;
    }

    /**
     * @param string $longOutput
     * @return $this
     */
    protected function setLongOutput($longOutput) {
        $this->longOutput = $longOutput;
        return $this;
    }

    /**
     * @param string $longOutput
     * @param bool $addNewLine
     * @return $this
     */
    protected function appendLongOutput($longOutput, $addNewLine = true) {
        $this->longOutput .= $longOutput;
        if ($addNewLine) {
            $this->longOutput .= PHP_EOL;
        }
        return $this;
    }

    /**
     * @param PerfdataGauge $PerfdataGauge
     * @return $this
     */
    protected function addPerfdata(PerfdataGauge $PerfdataGauge) {
        $this->perfdata[] = $PerfdataGauge;
        return $this;
    }

    /**
     * @return int
     */
    public function getStatuscode() {
        return $this->statusCode;
    }

    /**
     * @param bool $doStatusPrefix
     * @return string
     */
    public function getOutput($doStatusPrefix = true) {
        if ($doStatusPrefix) {
            return NagiosService::prefixOutput($this->output, $this->statusCode);
        }

        return $this->output;
    }

    /**
     * @return string
     */
    public function getLongOutput() {
        return $this->longOutput;
    }

    /**
     * @return string
     */
    public function getPerfdataSerialized() {
        if (empty($this->perfdata)) {
            return '';
        }

        $perfdataStrings = [];

        foreach ($this->perfdata as $Gauge) {
            /** @var PerfdataGauge $Gauge */
            $perfdataString = sprintf(
                '%s=%s%s;%s;%s;%s;%s',
                $Gauge->getLabel(),
                $Gauge->getValue(),
                $Gauge->getUnit(),
                $Gauge->getWarning(),
                $Gauge->getCritical(),
                $Gauge->getMin(),
                $Gauge->getMax()
            );
            $perfdataStrings[] = $perfdataString;
        }

        return implode(' ', $perfdataStrings);
    }

    /**
     * Returns false, even if $valueString is an empty string.
     * If $compareWith fits the defined range in $valueString, returns true.
     * If $unit is set, the values in $valueString will be calculated to B before comparing to $compareWith.
     *
     * @param $valueString
     * @param $compareWith
     * @param null $unit
     * @return bool
     */
    public function getRangeStatus($valueString, $compareWith, $unit = null) {    //$valueString -> (start ≤ end)
        if (trim($valueString) === '') {
            return false;
        }

        $compareWith = floatval($compareWith);

        $alertInsightRange = true;         //string starts with @
        $withExplicitComparison = false;    //string includes :
        $cValue1 = null;
        $cValue2 = null;

        if (strpos($valueString, '@') === 0) {
            $alertInsightRange = false;
            $valueString = substr($valueString, 1);
        }

        if (strpos($valueString, ':') !== false) {
            $withExplicitComparison = true;
            $vArray = explode(':', $valueString);
            if (isset($vArray[0]) && $vArray[0] !== '') {
                $cValue1 = $vArray[0];
            }

            if (isset($vArray[1]) && $vArray[1] !== '') {
                $cValue2 = $vArray[1];
            }

            if (strpos($valueString, ':') === 0) {    //:10 == 10
                $withExplicitComparison = false;
                $valueString = substr($valueString, 1);
            }
        }

        if ($withExplicitComparison && floatval($cValue1) && ($cValue2 == '' || $cValue2 == '~')) {   //10: || @10:
            if ($unit !== null) {
                $cValue1 = Byte::strToByte($cValue1 . $unit);
            }

            if ($alertInsightRange) {
                if ($compareWith < floatval($cValue1)) {
                    return true;
                }
            } else {
                if ($compareWith >= floatval($cValue1)) {
                    return true;
                }
            }
        } else if ($withExplicitComparison && floatval($cValue2) && $cValue1 == '~') {   //~:10 || @~:10
            if ($unit !== null) {
                $cValue2 = Byte::strToByte($cValue2 . $unit);
            }

            if ($alertInsightRange) {
                if ($compareWith > floatval($cValue2)) {
                    return true;
                }
            } else {
                if ($compareWith <= floatval($cValue2)) {
                    return true;
                }
            }
        } else if ($withExplicitComparison && floatval($cValue1) && floatval($cValue2)) {   //10:20 || @10:20
            if ($unit !== null) {
                $cValue1 = Byte::strToByte($cValue1 . $unit);
                $cValue2 = Byte::strToByte($cValue2 . $unit);
            }

            if ($alertInsightRange) {
                if ($compareWith < floatval($cValue1) || $compareWith > floatval($cValue2)) {
                    return true;
                }
            } else {
                if ($compareWith >= floatval($cValue1) && $compareWith <= floatval($cValue2)) {
                    return true;
                }
            }
        } else if (!$withExplicitComparison && floatval($valueString)) {      //10 || @10
            if ($unit !== null) {
                $valueString = Byte::strToByte($valueString . $unit);
            }

            if ($alertInsightRange) {
                if ($compareWith < 0 || $compareWith > floatval($valueString)) {
                    return true;
                }
            } else {
                if ($compareWith >= 0 && $compareWith <= floatval($valueString)) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * @param string $key in agent json
     * @return bool true if an error occurred
     */
    public function checkError($mainKey, $subKey = null) {
        if (!empty($subKey)) {
            if (isset($this->json[$mainKey][$subKey]['error'])) {
                $this->setOutput(sprintf(
                    'Agent error: %s',
                    $this->json[$mainKey][$subKey]['error']
                ));
                $this->setStatuscode(NagiosService::STATUS_CRITICAL);
                return true;
            }
        }

        if (isset($this->json[$mainKey]['error'])) {
            $this->setOutput(sprintf(
                'Agent error: %s',
                $this->json[$mainKey]['error']
            ));
            $this->setStatuscode(NagiosService::STATUS_CRITICAL);
            return true;
        }
        return false;
    }

}
